/* $Id: lb.h,v 1.11 2005/08/02 19:45:32 jonathan Exp $
 * Jonathan Ledlie, Harvard University.
 * Copyright 2005.  All rights reserved.
 */

#ifndef LB_H
#define LB_H

#include <stdio.h>
#include <stdlib.h>
#ifdef LINUX
#include <getopt.h>
#endif
#include <sys/types.h>
#include <unistd.h>
#include <algorithm>
#include <vector>
#include "math_util.h"
#include "error.h"
#include "distributions.h"
#include "server.h"

class Statistics {
 private:
  Long startTime;
  Long psSum;
  Long vsSum;
  Long queryMsgCount;
  Long maintMsgCount;
  Long queryCount;
  Long querySuccess;
  Long splitVsCount;
  Long transferVsSuccessCount;
  Long transferVsCount;
  Long deleteVsCount;
  Long createVsCount;
  Long vsAct;
  Long birthCount;
  Long deathCount;
  Long hopSum;
  Long hopSrcDstSum;
  Long reachableAttemptCount;
  Long reachableSuccessCount;

 public:
  Statistics ();
  ~Statistics ();
  void print (FILE *fp);
  void clear ();
  void birth (int act);
  void death (int act);
  void psUp (int currPsUp);
  void vsUp (int currVsUp);
  void queryMsg (int msg);
  void maintMsg (int msg);
  void query (bool successs, int hops);
  void hopSrcDst (bool reachable, int hopCount);
  void add (Statistics *s);
  void splitVs ();
  void transferVs (bool succ);
  void deleteVs ();
  void createVs ();
};

void oracleAllocateVsUniform (Distribution *keyDist,
			      PhysicalServer *ps);
void oracleAllocateVsZipf (Distribution *keyDist,
			   PhysicalServer *ps);

void printLinks (PhysicalServer *ps, FILE *fp, bool doPrintLinks);
void printVs (FILE *fp);
void printPs (PhysicalServer *ps, FILE *fp);
double getTargetMismatch (PhysicalServer *ps);
void printArcs (PhysicalServer *ps);

void openChurnFile (char *filename);
FILE* openOutputFile (char *prefix, char *suffix);
char nextEvent (int &nodeid);
void closeChurnFile ();

int initNodes (char *filename, PhysicalServer *&ps, int &initialVsPerNode);
void deleteNodes (PhysicalServer *ps);

extern int** queryMsg;
extern int** maintMsg;

int methodToCode (int idChoice, int initialVsPerNode, char activeLBmethod);

#endif
