/* $Id: lb_aux.cc,v 1.17 2005/08/02 19:45:32 jonathan Exp $
 * Jonathan Ledlie, Harvard University.
 * Copyright 2005.  All rights reserved.
 */

#include "lb.h"

#ifdef DEBUG
bool debug;
#endif

FILE *traceFP = NULL;
static FILE *churnFP;
int nextEventRound = -1;
char nextEventAction;
int nextEventNodeId;

int** queryMsg;
int** maintMsg;
int totalMsgCount = 0;
double lowerSlack;
double upperSlack;
int cRound;
int idChoice;
int maxVsPerNode;
bool oracle;
bool dampeningLimitCrDel = false;
bool dampeningLimitIds = false;


// Note that this prints routed msgs whether or not they succeeded
void printLinks (PhysicalServer *ps, FILE *fp, bool doPrintLinks) {
  for (int i = 0; i < psCount; i++) {
    for (int j = 0; j < psCount; j++) {
      if (doPrintLinks) fprintf (fp, "%d -> %d %d %d\n", i, j, queryMsg[i][j], maintMsg[i][j]);
      ps[i].incrMaintMsgOut (maintMsg[i][j]);
      ps[j].incrMaintMsgIn (maintMsg[i][j]);
      ps[i].incrQueryMsgOut (queryMsg[i][j]);
      ps[j].incrQueryMsgIn (queryMsg[i][j]);
    }
  }
}

void printPs (PhysicalServer *ps,  FILE *fp) {
  for (int i = 0; i < psCount; i++) {
    ps[i].print (fp);
  }
}

double getTargetMismatch (PhysicalServer *ps) {
  double sum = 0.;
  int upCount = 0;
  for (int i = 0; i < psCount; i++) {
    if (ps[i].isAlive()) {
      sum += fabs (ps[i].getWorkPerRound()-ps[i].getUpperTarget());
      upCount++;
    }
  }
  return sum/(double)upCount;
}

void printVs (FILE *fp) {
  int count = 0;
  for (map<double,VirtualServer*>::iterator p = vServers.begin();
       p != vServers.end(); p++) {
    count++;
    map<double,VirtualServer*>::iterator nextStep = p;
    nextStep++;
    if (nextStep == vServers.end())
      nextStep = vServers.begin();
    double distance = keyDistance (p->first,nextStep->first);

    fprintf (fp, "%d %d %f %f\n", count, p->second->getRootPs()->getId(),
	     p->first, distance);
  }
}

void printArcs (PhysicalServer *ps, int psCount) {
  /*
  vector<double> arcs;
  int upCount = 0;
  for (int i = 0; i < psCount; i++) {
    //    debug = true;
    if (ps[i].isAlive()) {
      //      double wcr = ps[i].getWorkToCapacityRatio ();
      arcs.push_back (ps[i].getArcLength());
      //      arcs.push_back (wcr);
      upCount++;
    }
  }
  sort(arcs.begin(), arcs.end());  
  for (int i = 0; i < upCount; i++) {
    printf ("%d %f\n", i, arcs[i]);
  }
  */
}


int initNodes (char *filename, PhysicalServer *&ps, int &initialVsPerNode) {
  int psCount = 0;
  FILE *fp;
  vector<int> capacity;
  if ((fp = fopen(filename, "r")) == NULL) {
    printf("%s: file open error.\n", filename);
    exit (-1);
  }

  // node-id capacity
  int id, cap;
  while (fscanf (fp, "%d %d\n", &id, &cap) > 0) {
    if (capacity.size() <= id) {
      capacity.resize (id+1);
    }
    capacity[id] = cap;
    psCount++;   
  }
  fclose (fp);

  ps = new PhysicalServer[psCount];
  ASSERT (ps);

  for (int i = 0; i < psCount; i++) {
    ps[i].setNode (i, (double)(capacity[i]));
  }

  queryMsg = new int* [psCount];
  maintMsg = new int* [psCount];
  for (int i = 0; i < psCount; i++) {
    queryMsg[i] = new int [psCount];
  }
  for (int i = 0; i < psCount; i++) {
    maintMsg[i] = new int [psCount];
  }
  for (int i = 0; i < psCount; i++) {
    for (int j = 0; j < psCount; j++) {
      queryMsg[i][j] = 0;
      maintMsg[i][j] = 0;
    }
  }

  initStep (psCount/2 * initialVsPerNode);

  return psCount;
}

void deleteNodes (PhysicalServer *ps) {
  delete [] ps;

  for (int i = 0; i < psCount; i++) {
    int* deleteMe = queryMsg[i];
    delete [] deleteMe;
    deleteMe = maintMsg[i];
    delete [] deleteMe;
  }
  delete [] queryMsg;
  delete [] maintMsg;
}

void openChurnFile (char *filename) {
  if ((churnFP = fopen(filename, "r")) == NULL) {
    printf("%s: file open error.\n", filename);
    exit (-1);
  }
}

char nextEvent (int &nodeid) {
  int round;
  char action;
  int node;

  if (nextEventRound > cRound) {
    return ' ';
  }

  if (nextEventRound == cRound) {
    nextEventRound = -1;
    nodeid = nextEventNodeId;
    return nextEventAction;
  }

  if (fscanf (churnFP, "%d %c %d\n", &round, &action, &node) <= 0) {
    return 'q';
  } else {
    ASSERT (round >= cRound);
    if (node > psCount || node < 0) {
      printf ("node %d, psCount %d, round %d\n", node, psCount, round);
      ASSERT (node < psCount && node >= 0);
    }

    if (round == cRound) {
      nodeid = node;
      return action;
    } else {
      nextEventRound = round;
      nextEventAction = action;
      nextEventNodeId = node;
    }
  }
  return ' ';
}

void closeChurnFile () {
  // for some reason, fclose hangs.
  //  fclose (churnFP);
}

FILE* openOutputFile (char *prefix, char *suffix) {
  FILE *fp;
  char *filename;
  int fileLength = strlen(prefix)+strlen(suffix)+1;

  filename = new char [fileLength];
  memset (filename,0,fileLength);
  sprintf (filename, "%s%s", prefix, suffix);
  if ((fp = fopen(filename, "w")) == NULL) {
    printf("%s: file open error.\n", filename);
    exit (-1);
  }
  delete [] filename;
  return fp;
}

int methodToCode (int idChoice, int initialVsPerNode, char activeLBmethod) {
  int b = 0;
  if (activeLBmethod == '-')
    b = 1;
  else if (activeLBmethod == 'k')
    b = 2;
  else if (activeLBmethod == 't')
    b = 3;
  else if (activeLBmethod == 'p')
    b = 4;
  return b;
}

const int MAX_QUERY_SUCCESS_LIST_SIZE = 20;
void initializePreviousQuerySuccess (deque<bool> &previousQuerySuccess) {
  for (int i = 0; i < MAX_QUERY_SUCCESS_LIST_SIZE; i++) {
    previousQuerySuccess.push_front (true);
  }
}

void addToPreviousQuerySuccess (deque<bool> &previousQuerySuccess,
                                bool routeSuccess) {
  previousQuerySuccess.pop_back();
  previousQuerySuccess.push_front (routeSuccess);
}

double avgOfPreviousQuerySuccess (deque<bool> &previousQuerySuccess,
                                  int sizeOfAvg) {
  double sum = 0.;
  ASSERT (sizeOfAvg <= MAX_QUERY_SUCCESS_LIST_SIZE);
  for (int i = 0; i < sizeOfAvg; i++) {
    if (previousQuerySuccess[i]) 
      sum += 1.;
  }
  return sum/sizeOfAvg;
}
